﻿using System.Reflection;
using UnityEditor;
using UnityEngine;

namespace Magify
{
    [CustomPropertyDrawer(typeof(MagifySettingsOptionalAttribute))]
    internal class MagifySettingsOptionalAttributeEditor : PropertyDrawer
    {
#if MAGIFY_ADVANCED
        private const bool HideAdvanced = false;
#else
        private const bool HideAdvanced = true;
#endif

        public override void OnGUI(Rect position, SerializedProperty property, GUIContent label)
        {
            using (property)
            {
                var header = ((MagifySettingsOptionalAttribute) attribute).Header;
                if (fieldInfo.IsStatic) return;
                var isArray = fieldInfo.FieldType.IsArray;

                if (!HideAdvanced && !isArray)
                {
                    if (header != null)
                    {
                        EditorGUILayout.BeginVertical();
                        position.y += EditorGUIUtility.singleLineHeight;
                        GUI.Label(position, header, EditorStyles.boldLabel);
                        GUILayout.Space(16);
                        EditorGUILayout.PropertyField(property, label);
                        EditorGUILayout.EndVertical();
                    }
                    else
                    {
                        EditorGUI.PropertyField(position, property, label);
                    }
                }

                if (isArray)
                {
                    EditorGUI.PropertyField(position, property, label);
                }
            }
        }

        public override float GetPropertyHeight(SerializedProperty property, GUIContent label)
        {
            if (!HideAdvanced && !fieldInfo.FieldType.IsArray)
            {
                return EditorGUI.GetPropertyHeight(property);
            }
            if (fieldInfo.FieldType.IsArray)
            {
                return EditorGUI.GetPropertyHeight(property);
            }
            return -EditorGUIUtility.standardVerticalSpacing;
        }
    }
}