﻿using System;
using System.Globalization;
using System.Threading;
using Cysharp.Threading.Tasks;
using UnityEngine;
using UnityEngine.Networking;

namespace Magify
{
    internal class MicrosoftServerTimeProvider : IServerTimeProvider
    {
        private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Network);

        public async UniTask<DateTime?> TryGetServerTime(CancellationToken cancellationToken)
        {
            var request = UnityWebRequest.Get("https://www.microsoft.com");
            await request.SendWebRequest().WithCancellation(cancellationToken);

            if (request.result is not UnityWebRequest.Result.Success)
            {
                MagifyLogger.Get().Log($"Failed to get server time from Microsoft with result: {request.result}, and error: {request.error}");
                return null;
            }

            try
            {
                var date = request.GetResponseHeader("date");
                _logger.Log($"Response time from Microsoft: {date}");
                var parsed = DateTime.ParseExact(date,
                    "ddd, dd MMM yyyy HH:mm:ss 'GMT'",
                    CultureInfo.InvariantCulture.DateTimeFormat,
                    DateTimeStyles.AssumeUniversal);
                _logger.Log($"Parsed time from Microsoft: UTC: {parsed.ToUniversalTime()}, Local: {parsed.ToLocalTime()}");
                return parsed.ToUniversalTime();
            }
            catch (Exception e)
            {
                _logger.LogError($"Exception during parsing server time from Microsoft: {e.Message}");
                _logger.LogException(e);
            }

            return null;
        }
    }
}